import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import 'oowoo_options.dart';

@immutable
class OOWOOThemeData with Diagnosticable {
  factory OOWOOThemeData({
    Color? bottomSheetGripColor,
    Color? bottomSheetBacgroudColor,
    Color Function()? basePlaceholderShimmer1,
    Color Function()? basePlaceholderShimmer2,
    Widget Function(Widget)? chatDetailBackgroundBuilder,
    Widget Function(Widget child, dynamic data, dynamic timestamp)?
        chatDetailContentActionBuilder,
    Widget Function(bool enableButtonBack, dynamic state)?
        chatDetailProfileBuilder,
    Widget Function(dynamic state, TextEditingController controller,
            FocusNode node, Function() onSend)?
        chatDetailInputBuilder,
    double? chatBubbleMaxWidth,
    EdgeInsets? chatBubbleMarginContent,
    TextStyle? chatTextStyleMessageState,
    Color? chatIconColorMessageState,
    Widget Function(String content, bool isMe, Map data, dynamic indexOfGroup)?
        chatTextContentBuilder,
    Widget Function(String content, bool isMe, Map data, dynamic indexOfGroup)?
        chatAudioContentBuilder,
    Color? chatBackgroundColorDetail,
    double? chatBubbleMaxWidthAppointment,
    Widget Function(Widget seenState, String thumbnail, String fullname,
            List<Widget> messages, bool isMe)?
        chatOwnerBubbleBuilder,
    Widget Function(bool chatListLoading, bool chatDetailLoading)?
        chatDetailLoadingBuilder,
    Widget Function(String asset, int count)? chatReactionIconBuilder,
    Widget Function(Widget child, Widget reaction, bool isMe)?
        chatReactionWrapBuilder,
    Widget Function(Map data, String imageUrl, String fullName)?
        chatTextReplyMessageBuilder,
    bool? enableReplyMessageOutsideBubble,
    Function(ImageProvider imageProvider, Map data)? chatOnOpenImage,
    List<Widget> Function(List messages)? chatDetailGroupedMessagesBuilder,
    OOWOOAuthOptions? authOptions,
    OOWOOAuthMobileOptions? authMobileOptions,
    OOWOOWidgetSettingOptions? widgetSettingOptions,
  }) {
    return OOWOOThemeData.raw(
      bottomSheetGripColor: bottomSheetGripColor,
      bottomSheetBacgroudColor: bottomSheetBacgroudColor,
      basePlaceholderShimmer1: basePlaceholderShimmer1,
      basePlaceholderShimmer2: basePlaceholderShimmer2,
      chatDetailProfileBuilder: chatDetailProfileBuilder,
      chatDetailInputBuilder: chatDetailInputBuilder,
      chatBackgroundColorDetail: chatBackgroundColorDetail,
      chatBubbleMaxWidth: chatBubbleMaxWidth,
      chatBubbleMaxWidthAppointment: chatBubbleMaxWidthAppointment,
      chatBubbleMarginContent: chatBubbleMarginContent,
      chatTextStyleMessageState: chatTextStyleMessageState,
      chatIconColorMessageState: chatIconColorMessageState,
      chatTextContentBuilder: chatTextContentBuilder,
      chatAudioContentBuilder: chatAudioContentBuilder,
      chatOwnerBubbleBuilder: chatOwnerBubbleBuilder,
      chatReactionIconBuilder: chatReactionIconBuilder,
      chatReactionWrapBuilder: chatReactionWrapBuilder,
      chatDetailBackgroundBuilder: chatDetailBackgroundBuilder,
      chatDetailContentActionBuilder: chatDetailContentActionBuilder,
      chatTextReplyMessageBuilder: chatTextReplyMessageBuilder,
      enableReplyMessageOutsideBubble: enableReplyMessageOutsideBubble,
      chatDetailLoadingBuilder: chatDetailLoadingBuilder,
      chatOnOpenImage: chatOnOpenImage,
      chatDetailGroupedMessagesBuilder: chatDetailGroupedMessagesBuilder,
      authOptions: authOptions,
      authMobileOptions: authMobileOptions,
      widgetSettingOptions: widgetSettingOptions,
    );
  }

  const OOWOOThemeData.raw({
    this.chatDetailBackgroundBuilder,
    this.chatDetailContentActionBuilder,
    this.bottomSheetGripColor,
    this.basePlaceholderShimmer1,
    this.basePlaceholderShimmer2,
    this.bottomSheetBacgroudColor,
    this.chatDetailProfileBuilder,
    this.chatDetailInputBuilder,
    this.chatBubbleMarginContent,
    this.chatBubbleMaxWidth,
    this.chatBubbleMaxWidthAppointment,
    this.chatTextContentBuilder,
    this.chatAudioContentBuilder,
    this.chatTextStyleMessageState,
    this.chatIconColorMessageState,
    this.chatBackgroundColorDetail,
    this.chatOwnerBubbleBuilder,
    this.chatReactionIconBuilder,
    this.chatReactionWrapBuilder,
    this.chatTextReplyMessageBuilder,
    this.enableReplyMessageOutsideBubble,
    this.chatDetailLoadingBuilder,
    this.chatOnOpenImage,
    this.chatDetailGroupedMessagesBuilder,
    this.authOptions,
    this.authMobileOptions,
    this.widgetSettingOptions,
  });

  final Color? bottomSheetGripColor;
  final Color? bottomSheetBacgroudColor;

  final Color Function()? basePlaceholderShimmer1;
  final Color Function()? basePlaceholderShimmer2;

  //Login
  final OOWOOAuthOptions? authOptions;
  final OOWOOAuthMobileOptions? authMobileOptions;

  //Widget Settings
  final OOWOOWidgetSettingOptions? widgetSettingOptions;

  //Chat
  final double? chatBubbleMaxWidth;
  final double? chatBubbleMaxWidthAppointment;

  //UI chat detail screen
  final Widget Function(bool chatListLoading, bool chatDetailLoading)?
      chatDetailLoadingBuilder;
  final Widget Function(Widget)? chatDetailBackgroundBuilder;
  final Color? chatBackgroundColorDetail;
  final Widget Function(bool enableButtonBack, dynamic state)?
      chatDetailProfileBuilder;
  final Widget Function(dynamic state, TextEditingController controller,
      FocusNode node, Function() onSend)? chatDetailInputBuilder;

  //Message state styles
  final TextStyle? chatTextStyleMessageState;
  final Color? chatIconColorMessageState;

  //Message content styles
  final EdgeInsets? chatBubbleMarginContent;
  final List<Widget> Function(List messages)? chatDetailGroupedMessagesBuilder;
  final Widget Function(Widget child, dynamic data, dynamic timestamp)?
      chatDetailContentActionBuilder;
  final Widget Function(
          String content, bool isMe, Map data, dynamic indexOfGroup)?
      chatTextContentBuilder;
  final Widget Function(
          String content, bool isMe, Map data, dynamic indexOfGroup)?
      chatAudioContentBuilder;
  final Widget Function(Map data, String imageUrl, String fullName)?
      chatTextReplyMessageBuilder;
  final bool? enableReplyMessageOutsideBubble;

  //Owner infos and messages display
  final Widget Function(Widget seenState, String thumbnail, String fullname,
      List<Widget> messages, bool isMe)? chatOwnerBubbleBuilder;

  //Reaction icon display builder
  final Widget Function(String asset, int count)? chatReactionIconBuilder;
  final Widget Function(Widget child, Widget reaction, bool isMe)?
      chatReactionWrapBuilder;

  //Open message type image
  final Function(ImageProvider imageProvider, Map data)? chatOnOpenImage;
}

class OOWOOTheme extends InheritedTheme {
  const OOWOOTheme({Key? key, required this.data, required this.child})
      : super(key: key, child: child);

  final OOWOOThemeData data;

  @override
  final Widget child;

  static OOWOOThemeData of(BuildContext context) {
    final OOWOOTheme? sfCalendarTheme =
        context.dependOnInheritedWidgetOfExactType<OOWOOTheme>();
    return sfCalendarTheme!.data;
  }

  @override
  bool updateShouldNotify(OOWOOTheme oldWidget) => data != oldWidget.data;

  @override
  Widget wrap(BuildContext context, Widget child) {
    final OOWOOTheme? ancestorTheme =
        context.findAncestorWidgetOfExactType<OOWOOTheme>();
    return identical(this, ancestorTheme)
        ? child
        : OOWOOTheme(data: data, child: child);
  }
}
