import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_keyboard_visibility/flutter_keyboard_visibility.dart';
import 'package:get/get.dart';
import 'package:lottie/lottie.dart';
import 'package:oowoo_core/oowoo_core.dart';

import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:oowoo_core/network_resources/resources.dart';
import 'package:oowoo_core/network_resources/model/model.dart';
import 'package:oowoo_core/widgets/widget_animated_icon.dart';
import 'bloc/user_settings_bloc.dart';
import '../settings_widgets/styles.dart';
import '../settings_widgets/widget_app_pass_pattern.dart';
import '../settings_widgets//widget_datetime_format_setting.dart';
import '../settings_widgets//widget_languages_setting.dart';
import '../settings_widgets//widget_input.dart';
import '../settings_widgets//widget_option.dart';

class SettingScreen extends StatefulWidget {
  const SettingScreen({Key? key}) : super(key: key);

  @override
  _SettingScreenState createState() => _SettingScreenState();
}

class _SettingScreenState extends State<SettingScreen> {
  late UserSettingsBloc bloc = Get.find<UserSettingsBloc>();
  final KeyboardVisibilityController _keyboardVisibilityController =
      KeyboardVisibilityController();
  StreamSubscription? subscription;

  TextEditingController nickNameController = TextEditingController();
  TextEditingController firstNameController = TextEditingController();
  TextEditingController lastNameController = TextEditingController();
  TextEditingController emailController = TextEditingController();
  TextEditingController phoneController = TextEditingController();

  FocusNode nickNameFocusNode = FocusNode();
  FocusNode firstNameFocusNode = FocusNode();
  FocusNode lastNameFocusNode = FocusNode();
  FocusNode emailFocusNode = FocusNode();
  FocusNode phoneNode = FocusNode();

  AnimateIconController controllerIconExpanded = AnimateIconController();
  GlobalKey<ExpandableNotifierState> expandableKey =
      GlobalKey<ExpandableNotifierState>();

  onSubmitted() {}

  onToggle() {
    oowooHaptic();
    expandableKey.currentState?.controller?.toggle();
    if (controllerIconExpanded.isStart()) {
      controllerIconExpanded.animateToEnd();
    } else if (controllerIconExpanded.isEnd()) {
      controllerIconExpanded.animateToStart();
    }
  }

  @override
  void initState() {
    bloc.add(const UserInfoFetch());
    super.initState();
    setEditControllerValue(appPrefs.loginUser!);
    subscription = _keyboardVisibilityController.onChange.listen((status) {
      if (!status && Get.isBottomSheetOpen != true) {
        ProInfo user = appPrefs.loginUser!;
        user.displayName = nickNameController.text.trim();
        user.firstName = firstNameController.text.trim();
        user.lastName = lastNameController.text.trim();
        user.email = emailController.text.trim();
        bloc.add(UserInfoSave(user));
      }
    });
  }

  @override
  void dispose() {
    super.dispose();
    nickNameController.dispose();
    firstNameController.dispose();
    lastNameController.dispose();
    emailController.dispose();
    firstNameFocusNode.dispose();
    lastNameFocusNode.dispose();
    emailFocusNode.dispose();
    subscription?.cancel();
  }

  void setEditControllerValue(ProInfo user) {
    nickNameController.text = user.displayName ?? '';
    firstNameController.text = user.firstName ?? '';
    lastNameController.text = user.lastName ?? '';
    phoneController.text = user.phone1 ?? '';
    emailController.text = user.email ?? '';
  }

  @override
  Widget build(BuildContext context) {
    return KeyboardVisibilityProvider(
      controller: _keyboardVisibilityController,
      child: KeyboardDismissOnTap(
        child: Scaffold(
          resizeToAvoidBottomInset: true,
          backgroundColor: appColors.appBackground,
          body: BlocConsumer<UserSettingsBloc, UserSettingsState>(
              bloc: bloc,
              listener: (BuildContext context, state) {
                Timer(const Duration(milliseconds: 500),
                    () => setEditControllerValue(state.userInfo));
              },
              builder: (BuildContext context, state) {
                return Column(
                  children: [
                    Center(
                      child: StreamBuilder(
                          stream: appPrefs.watch('user_info'),
                          builder: (_, snapshot) {
                            if (appPrefs.loginUser == null) {
                              return const SizedBox();
                            }
                            return Column(
                              mainAxisSize: MainAxisSize.min,
                              children: [
                                SizedBox(
                                  height:
                                      24 + MediaQuery.of(context).padding.top,
                                ),
                                Row(
                                  children: [
                                    const Spacer(),
                                    GestureDetector(
                                      onTap: () {
                                        oowooHaptic();
                                        bloc.add(const UserChangeAvatar());
                                      },
                                      child: Stack(
                                        alignment: Alignment.center,
                                        children: [
                                          WidgetAvatar(
                                            radius1: 50,
                                            radius2: 47,
                                            radius3: 43,
                                            imageUrl:
                                                appPrefs.loginUser!.thumbnail,
                                            borderColor: appColors.text,
                                          ),
                                          if (state.avatarUploading)
                                            Lottie.asset(
                                                assetlottie('changePicture'),
                                                width: 80),
                                        ],
                                      ),
                                    ),
                                    Expanded(
                                        child: Align(
                                      alignment: Alignment.centerRight,
                                      child: IconButton(
                                        onPressed: () {
                                          oowooHaptic();
                                          Get.back();
                                        },
                                        icon: Icon(
                                          Icons.arrow_back_ios_rounded,
                                          color: textColorSetting,
                                        ),
                                      ),
                                    ))
                                  ],
                                ),
                                const SizedBox(
                                  height: 12,
                                ),
                                Text(
                                  '${appPrefs.loginUser!.displayName}',
                                  textAlign: TextAlign.center,
                                  style: robotoRegular,
                                ),
                              ],
                            );
                          }),
                    ),
                    const SizedBox(
                      height: 12,
                    ),
                    Expanded(
                      child: SingleChildScrollView(
                        padding: const EdgeInsets.symmetric(horizontal: 20),
                        physics: const BouncingScrollPhysics(),
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          mainAxisSize: MainAxisSize.min,
                          children: [
                            _buildInfo(state),
                            const SizedBox(
                              height: 20,
                            ),
                            _buildOptions(state),
                            SizedBox(
                              height: MediaQuery.of(context).padding.bottom,
                            ),
                          ],
                        ),
                      ),
                    ),
                  ],
                );
              }),
        ),
      ),
    );
  }

  Widget _buildInfo(UserSettingsState state) {
    return Column(
      children: [
        const SizedBox(
          height: 8,
        ),
        Card(
          color: appColors.element,
          margin: EdgeInsets.zero,
          clipBehavior: Clip.antiAlias,
          shape:
              RoundedRectangleBorder(borderRadius: BorderRadius.circular(12)),
          child: Padding(
            padding: const EdgeInsets.symmetric(vertical: 20, horizontal: 28),
            child: ExpandableNotifier(
              key: expandableKey,
              initialExpanded: false,
              child: ExpandablePanel(
                theme: ExpandableThemeData(
                  headerAlignment: ExpandablePanelHeaderAlignment.center,
                  tapBodyToExpand: false,
                  tapBodyToCollapse: false,
                  tapHeaderToExpand: false,
                  hasIcon: false,
                ),
                header: GestureDetector(
                  onTap: onToggle,
                  child: Container(
                    color: Colors.transparent,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Row(
                          children: [
                            Expanded(
                              child: Text(
                                'personalinfors'.tr,
                                style: latoBold.copyWith(
                                    color: text, fontSize: 14),
                              ),
                            ),
                            AnimateIcons(
                              isButton: false,
                              startIcon: Icons.keyboard_arrow_down_outlined,
                              endIcon: Icons.keyboard_arrow_up_outlined,
                              controller: controllerIconExpanded,
                              onStartIconPress: () {
                                return true;
                              },
                              onEndIconPress: () {
                                return true;
                              },
                              duration: const Duration(milliseconds: 350),
                              startIconColor: textColorSetting,
                              endIconColor: textColorSetting,
                              clockwise: false,
                            ),
                          ],
                        ),
                        const SizedBox(
                          height: 4,
                        ),
                        Text(
                          'oowooisatrustyplatform'.tr +
                              '\n' +
                              'toprotectyourprivacy'.tr,
                          style: latoLight.copyWith(
                              fontSize: 10, color: textColorSetting),
                        ),
                      ],
                    ),
                  ),
                ),
                collapsed: const SizedBox(),
                expanded: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    const SizedBox(
                      height: 24,
                    ),
                    WidgetSettingInput(
                      node: nickNameFocusNode,
                      next: firstNameFocusNode,
                      title: 'nickname'.tr,
                      controller: nickNameController,
                      onSubmitted: onSubmitted,
                    ),
                    WidgetSettingInput(
                      node: firstNameFocusNode,
                      next: lastNameFocusNode,
                      title: 'firstname'.tr,
                      controller: firstNameController,
                      onSubmitted: onSubmitted,
                    ),
                    WidgetSettingInput(
                      node: lastNameFocusNode,
                      next: emailFocusNode,
                      title: 'lastname'.tr,
                      controller: lastNameController,
                      onSubmitted: onSubmitted,
                    ),
                    WidgetSettingInput(
                      node: phoneNode,
                      readOnly: true,
                      title: 'phonenumber'.tr,
                      controller: phoneController,
                      onSubmitted: onSubmitted,
                    ),
                    GestureDetector(
                      onTap: () {
                        oowooHaptic();
                        //   showMaterialModalBottomSheet(
                        //       context: context,
                        //       backgroundColor: Colors.transparent,
                        //       builder: (_) => const WidgetEmailUpdateSetting());
                      },
                      child: Container(
                        color: Colors.transparent,
                        child: IgnorePointer(
                          ignoring: true,
                          child: WidgetSettingInput(
                            node: emailFocusNode,
                            title: 'email'.tr,
                            controller: emailController,
                            onSubmitted: onSubmitted,
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildOptions(UserSettingsState state) {
    return Column(
      children: [
        Card(
          color: appColors.element,
          margin: EdgeInsets.zero,
          clipBehavior: Clip.antiAlias,
          shape:
              RoundedRectangleBorder(borderRadius: BorderRadius.circular(12)),
          child: Padding(
            padding: const EdgeInsets.symmetric(vertical: 20, horizontal: 16),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                WidgetSettingItemOption(
                  icon: 'password',
                  title: 'password'.tr,
                  hint: 'addapasswordtounlok'.tr,
                  onTap: () async {
                    oowooHaptic();
                    showBottomSheetTransparent(const WidgetAppPassPattern(),
                        isScrollControlled: false, enableDrag: false);
                  },
                ),
                const WidgetSettingSparater(),
                WidgetSettingItemOption(
                  icon: 'language',
                  title: widgetSettingOptions.getLocale().languageCode.tr,
                  hint: 'chooseyourfavoritelanguage'.tr,
                  onTap: () {
                    oowooHaptic();
                    Get.bottomSheet(const WidgetLanguagesSetting());
                  },
                ),
                const WidgetSettingSparater(),
                WidgetSettingItemOption(
                  icon: 'datetime',
                  title: 'dateformat'.tr,
                  hint: 'thetimeanddataformatvary'.tr,
                  onTap: () async {
                    oowooHaptic();
                    await Get.bottomSheet(const WidgetDateFormat());
                    OOWOOConfig.of(Get.context!)
                        .setLocale(appPrefs.languageCode);
                  },
                ),
                const WidgetSettingSparater(),
                WidgetSettingItemOption(
                  icon: 'time',
                  title: 'timeformat'.tr,
                  hint: 'thetimeanddataformatvary2'.tr,
                  onTap: () async {
                    oowooHaptic();
                    await Get.bottomSheet(const WidgetDateTimeFormat());
                    OOWOOConfig.of(Get.context!)
                        .setLocale(appPrefs.languageCode);
                  },
                ),
              ],
            ),
          ),
        ),
        const SizedBox(
          height: 20,
        ),
        Card(
          color: appColors.element,
          margin: EdgeInsets.zero,
          clipBehavior: Clip.antiAlias,
          shape:
              RoundedRectangleBorder(borderRadius: BorderRadius.circular(12)),
          child: Padding(
            padding: const EdgeInsets.symmetric(vertical: 16, horizontal: 16),
            child: GestureDetector(
              onTap: widgetSettingOptions.onLogout,
              child: Container(
                color: Colors.transparent,
                padding: const EdgeInsets.symmetric(horizontal: 10),
                child: Row(
                  children: [
                    const SizedBox(
                      width: 4,
                    ),
                    SizedBox(
                      width: 46,
                      child: Align(
                        alignment: Alignment.centerLeft,
                        child: Icon(
                          Icons.logout,
                          color: textColorSetting,
                        ),
                      ),
                    ),
                    Expanded(
                      child: Text(
                        'logout'.tr,
                        style: latoMedium.copyWith(fontSize: 14.0),
                      ),
                    )
                  ],
                ),
              ),
            ),
          ),
        ),
        const SizedBox(
          height: 65,
        )
      ],
    );
  }
}
