import 'package:oowoo_core/network_resources/resources.dart';
import 'package:oowoo_core/oowoo_core.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get/get.dart';
import 'package:image_cropper/image_cropper.dart';
import 'package:image_picker/image_picker.dart';

import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:oowoo_core/widgets/settings_widgets/styles.dart';
import 'package:oowoo_core/widgets/settings_widgets/widget_method_upload.dart';

part 'user_settings_event.dart';
part 'user_settings_state.dart';

class UserSettingsBloc extends Bloc<UserSettingsEvent, UserSettingsState> {
  final ImagePicker picker = ImagePicker();
  final AuthRepo authRepo = AuthRepo();
  UserSettingsBloc() : super(UserSettingsState.initial()) {
    on<UserInfoFetch>(initialFetch);
    on<UserInfoSave>(saveUserInfo);
    on<UserChangeAvatar>(userChangeAvatar);
  }

  initialFetch(event, Emitter<UserSettingsState> emit) async {
    UserModel? userInfo = await authRepo.getUserInfo();
    if (userInfo != null) {
      appPrefs.loginUser = userInfo;
      emit(state.update(userInfo: userInfo));
    } else {
      emit(state.update());
    }
  }

  saveUserInfo(UserInfoSave event, Emitter<UserSettingsState> emit) async {
    try {
      final result = await authRepo.updateUser(event.userInfo.toJson());
      if (result.status) {
        appPrefs.loginUser = event.userInfo;
        emit(state.update(userInfo: appPrefs.loginUser));
      } else {
        if (result.msg == MESSAGE_API_EXISTING_USER_EMAIL) {
          // AppUtils.showApiMessageError(ApiMessageErrorType.existingUserEmail);
        } else if (result.msg == MESSAGE_API_EXISTING_PRO_EMAIL) {
          // AppUtils.showApiMessageError(ApiMessageErrorType.existingProEmail);
        }
      }
    } catch (e) {
      appDebugPrint('saveUserInfo: $e');
      emit(state.update());
    }
  }

  userChangeAvatar(
      UserChangeAvatar event, Emitter<UserSettingsState> emit) async {
    try {
      final source = await Get.bottomSheet(const WidgetMethodUpload());
      if (source is! ImageSource) return;
      // appPrefs.enablePasscode = false;
      XFile? file = await picker.pickImage(source: source, imageQuality: 70);
      if (file != null) {
        var cropped = await ImageCropper().cropImage(
          sourcePath: file.path,
          aspectRatio: const CropAspectRatio(ratioX: 1, ratioY: 1),
          uiSettings: [
            AndroidUiSettings(
                toolbarTitle: 'Cropper'.tr,
                toolbarColor: widgetSettingOptions.mainColor(),
                activeControlsWidgetColor: appColors.text,
                toolbarWidgetColor: Colors.white,
                initAspectRatio: CropAspectRatioPreset.original,
                lockAspectRatio: true),
            IOSUiSettings(
              minimumAspectRatio: 1.0,
              aspectRatioLockEnabled: true,
              aspectRatioPickerButtonHidden: true,
            ),
          ],
        );
        String? avatarImageUrl;
        if (cropped != null) {
          emit(state.update(avatarUploading: true));
          avatarImageUrl = await authRepo.setThumbnailForUser(
              await cropped.readAsBytes(), cropped.path);
          var user = state.userInfo;
          user.thumbnail = avatarImageUrl;
          final result = await authRepo.updateUser(user.toJson());
          emit(state.update(avatarUploading: false));
          if (result.status == true) {
            state.userInfo.thumbnail = avatarImageUrl;
            appPrefs.loginUser = user;
            emit(state.update());
          }
        }
      }
      // appPrefs.enablePasscode = true;
    } catch (e) {
      appDebugPrint('userChangeAvatar: $e');
    }
  }
}
