import 'dart:async';
import 'dart:math';

import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:get/get.dart';
import 'package:local_auth/local_auth.dart';
import 'package:local_auth/error_codes.dart' as auth_error;
import 'package:lottie/lottie.dart';
import 'package:oowoo_core/oowoo_core.dart';
import 'styles.dart';

Color get blue => widgetSettingOptions.mainColor();
Color get red => hexColor('#EF4068');

final LocalAuthentication auth = LocalAuthentication();

class WidgetAppPassPattern extends StatefulWidget {
  const WidgetAppPassPattern({Key? key}) : super(key: key);

  @override
  State<WidgetAppPassPattern> createState() => _WidgetAppPassPatternState();
}

class _WidgetAppPassPatternState extends State<WidgetAppPassPattern> {
  double valueKey1 = 2;
  double valueKey2 = 3;
  late bool isCreateMode = appPrefs.appPassPattern == null;
  List<int> appPass1 = [];
  int step = 1;
  PassPatterState patternState = PassPatterState.drawing;

  @override
  Widget build(BuildContext context) {
    return widgetSettingOptions.bottomSheetBuilder(
      padding: const EdgeInsets.fromLTRB(32, 32, 32, 40),
      child: isCreateMode
          ? AnimatedSwitcher(
              duration: const Duration(milliseconds: 350),
              child: step == 1
                  ? _buildCreateStep1()
                  : step == 2
                      ? _buildCreateStep2()
                      : _buildCreateStep3(),
              layoutBuilder:
                  (Widget? currentChild, List<Widget> previousChildren) {
                List<Widget> children = previousChildren;
                if (currentChild != null) {
                  children = children.toList()..add(currentChild);
                }
                return Stack(
                  alignment: Alignment.center,
                  children: children,
                );
              },
              transitionBuilder: (Widget child, Animation<double> animation) {
                Animation<Offset> inAni;
                Animation<Offset> outAni;
                inAni = Tween<Offset>(
                        begin: const Offset(2.0, 0.0),
                        end: const Offset(0.0, 0.0))
                    .animate(animation);
                outAni = Tween<Offset>(
                        begin: const Offset(-2.20, 0.0),
                        end: const Offset(0.0, 0.0))
                    .animate(animation);

                return SlideTransition(
                  position:
                      (child.key as ValueKey).value < step ? outAni : inAni,
                  child: child,
                );
              })
          : _buildDeactive(),
    );
  }

  Widget _buildCreateStep1() {
    return Column(
      key: const ValueKey(1),
      children: [
        Lottie.asset(assetlottie('passwordON'), width: Get.width / 3),
        const SizedBox(
          height: 46,
        ),
        GestureDetector(
          onTap: () {
            oowooHaptic();
            setState(() {
              step = 2;
            });
          },
          child: Container(
            width: Get.width,
            padding: const EdgeInsets.symmetric(vertical: 16),
            decoration: BoxDecoration(
                color: blue, borderRadius: BorderRadius.circular(16)),
            child: Center(
              child: Text(
                'activethecodeprotection'.tr,
                style:
                    assistantBold.copyWith(color: Colors.white, fontSize: 16),
              ),
            ),
          ),
        ),
      ],
    );
  }

  Offset? offset;
  List<int> codes = [];
  _onPanStart(DragStartDetails event) => _clearCodes();

  _onPanUpdate(DragUpdateDetails event) =>
      setState(() => offset = event.localPosition);

  _onPanEnd(DragEndDetails event) {
    setState(() => offset = null);
    Timer(const Duration(milliseconds: 300), () {
      setState(() {
        if (codes.length >= 4) {
          patternState = PassPatterState.correct;
          appPass1 = List.from(codes);
        } else {
          patternState = PassPatterState.wrong;
        }
      });
      Timer(const Duration(milliseconds: 300), () {
        setState(() {
          if (codes.length >= 4) {
            step = 3;
          }
          codes.clear();
        });
      });
    });
  }

  _onPanEndConfirm(DragEndDetails event) {
    Function eq = const ListEquality().equals;
    setState(() => offset = null);
    Timer(const Duration(milliseconds: 300), () {
      if (mounted) {
        if (eq(codes, appPass1)) {
          patternState = PassPatterState.correct;
          appPrefs.appPassPattern = appPass1;
          oowooHaptic();
          Timer(const Duration(milliseconds: 300), () {
            Get.back();
          });
        } else {
          oowooHaptic();
          patternState = PassPatterState.wrong;
          valueKey2 = Random().nextDouble();
        }
        setState(() {});
      }
    });
  }

  _onSelect(int code) {
    if (codes.isEmpty || codes.last != code) {
      oowooHaptic();
      codes.add(code);
    }
  }

  _clearCodes() => setState(() {
        patternState = PassPatterState.drawing;
        codes = [];
        offset = null;
      });

  late double sizePainter = Get.width - 32 * 3;

  Widget _buildCreateStep2() {
    return Column(
      key: const ValueKey(2),
      children: [
        Text(
          'drawapattern'.tr,
          style: assistantMedium.copyWith(fontSize: 16),
        ),
        const SizedBox(
          height: 4,
        ),
        Text(
          'minimum4dots'.tr,
          style: assistantLight.copyWith(
              fontSize: 12,
              color:
                  patternState == PassPatterState.wrong ? red : appColors.text),
        ),
        const SizedBox(
          height: 16,
        ),
        SizedBox(
          height: sizePainter,
          child: GestureDetector(
            onPanStart: _onPanStart,
            onPanUpdate: _onPanUpdate,
            onPanEnd: _onPanEnd,
            child: CustomPaint(
              key: ValueKey(patternState),
              painter: _LockScreenPainter(
                  mainColor: patternState == PassPatterState.correct
                      ? blue
                      : patternState == PassPatterState.wrong
                          ? red
                          : appColors.text,
                  codes: codes,
                  offset: offset,
                  onSelect: _onSelect),
              size: Size.square(sizePainter),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildCreateStep3() {
    return Column(
      key: const ValueKey(3),
      children: [
        Text(
          'drawapattern'.tr,
          style: assistantMedium.copyWith(fontSize: 16),
        ),
        const SizedBox(
          height: 4,
        ),
        if (patternState == PassPatterState.wrong)
          Text(
            'wrongpattern'.tr,
            style: assistantLight.copyWith(fontSize: 12, color: red),
          )
        else
          Text(
            'minimum4dots'.tr,
            style: assistantLight.copyWith(fontSize: 12),
          ),
        const SizedBox(
          height: 16,
        ),
        SizedBox(
          height: sizePainter,
          child: GestureDetector(
            onPanStart: _onPanStart,
            onPanUpdate: _onPanUpdate,
            onPanEnd: _onPanEndConfirm,
            child: CustomPaint(
              painter: _LockScreenPainter(
                  mainColor: patternState == PassPatterState.correct
                      ? blue
                      : patternState == PassPatterState.wrong
                          ? red
                          : appColors.text,
                  codes: codes,
                  offset: offset,
                  onSelect: _onSelect),
              size: Size.square(sizePainter),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildDeactive() {
    return Column(
      children: [
        // FutureBuilder<bool>(
        //     future: auth.canCheckBiometrics,
        //     builder: (context, snapshot) {
        //       if (snapshot.connectionState != ConnectionState.done ||
        //           snapshot.data != true) {
        //         return const SizedBox();
        //       }
        //       return Row(
        //         children: [
        //           FlutterSwitch(
        //             width: 40.0,
        //             height: 22.0,
        //             toggleSize: 18.0,
        //             padding: 2.0,
        //             inactiveColor: Colors.transparent,
        //             activeColor: Colors.transparent,
        //             switchBorder: Border.all(
        //               color: appColors.text.withOpacity(
        //                   appPrefs.appPassBiometrics ? 1 : .5),
        //             ),
        //             value: appPrefs.appPassBiometrics,
        //             toggleColor: appColors.text.withOpacity(
        //                 appPrefs.appPassBiometrics ? 1 : .5),
        //             onToggle: (_) {
        //               setState(() {
        //                 appPrefs.appPassBiometrics = _;
        //               });
        //             },
        //           ),
        //           const SizedBox(
        //             width: 16,
        //           ),
        //           Text(
        //             'unlockwithfigperprint'.tr,
        //             style: assistantMedium.copyWith(
        //                 color: appColors.text.withOpacity(
        //                     appPrefs.appPassBiometrics ? 1 : .5)),
        //           ),
        //         ],
        //       );
        //     }),
        const SizedBox(
          height: 24,
        ),
        Lottie.asset(assetlottie('passwordOFF'), width: Get.width / 3),
        const SizedBox(
          height: 46,
        ),
        GestureDetector(
          onTap: () {
            oowooHaptic();
            appPrefs.appPassPattern = null;
            Get.back();
          },
          child: Container(
            width: Get.width,
            padding: const EdgeInsets.symmetric(vertical: 16),
            decoration: BoxDecoration(
                color: red, borderRadius: BorderRadius.circular(16)),
            child: Center(
              child: Text(
                'deactivecodeprotection'.tr,
                style:
                    assistantBold.copyWith(color: Colors.white, fontSize: 16),
              ),
            ),
          ),
        ),
      ],
    );
  }
}

class _LockScreenPainter extends CustomPainter {
  final int _total = 9;
  final int _col = 3;
  Size? size;

  final List<int> codes;
  final Offset? offset;
  final Function(int code) onSelect;
  final Color mainColor;

  _LockScreenPainter({
    required this.mainColor,
    required this.codes,
    required this.offset,
    required this.onSelect,
  });

  double get _sizeCode => size!.width / _col;

  Paint get _painter => Paint()
    ..color = Colors.white
    ..strokeWidth = 2.0;

  @override
  void paint(Canvas canvas, Size size) {
    this.size = size;

    for (var i = 0; i < _total; i++) {
      var _offset = _getOffetByIndex(i);

      var _radiusIn = _sizeCode / 2.0 * 0.2;
      _drawCircle2(canvas, _offset, _radiusIn, _getColorByIndex(i));

      var _radiusOut = _sizeCode / 2.0 * 0.6;
      _drawCircle(canvas, _offset, _radiusOut, _getColorByIndex2(i));

      var _pathGesture = _getCirclePath(_offset, _radiusOut);
      if (offset != null && _pathGesture.contains(offset!)) onSelect(i);
    }

    for (var i = 0; i < codes.length; i++) {
      var _start = _getOffetByIndex(codes[i]);
      if (i + 1 < codes.length) {
        var _end = _getOffetByIndex(codes[i + 1]);
        _drawLine(canvas, _start, _end);
      } else if (offset != null) {
        var _end = offset!;
        _drawLine(canvas, _start, _end);
      }
    }
  }

  Path _getCirclePath(Offset offset, double radius) {
    var _rect = Rect.fromCircle(radius: radius, center: offset);
    return Path()..addOval(_rect);
  }

  void _drawCircle2(Canvas canvas, Offset offset, double radius, color) {
    var _path = _getCirclePath(offset, radius);
    var _path2 = _getCirclePath(offset, radius + 3);
    var _painter = this._painter
      ..color = color
      ..style = PaintingStyle.fill;
    var _painter2 = this._painter
      ..color = Colors.black
      ..style = PaintingStyle.fill;
    canvas.drawPath(_path2, _painter2);
    canvas.drawPath(_path, _painter);
  }

  void _drawCircle(Canvas canvas, Offset offset, double radius, Color? color) {
    var _path = _getCirclePath(offset, radius);
    var _painter = this._painter
      ..color = color != null ? color.withOpacity(.25) : Colors.transparent
      ..style = PaintingStyle.fill;
    canvas.drawPath(_path, _painter);
  }

  void _drawLine(Canvas canvas, Offset start, Offset end) {
    var _painter = this._painter
      ..color = mainColor
      ..style = PaintingStyle.stroke
      ..strokeWidth = 2.0;
    var _path = Path();
    _path.moveTo(start.dx, start.dy);
    _path.lineTo(end.dx, end.dy);
    canvas.drawPath(_path, _painter);
  }

  Color _getColorByIndex(int i) {
    return codes.contains(i) ? mainColor : Colors.white;
  }

  Color? _getColorByIndex2(int i) {
    return codes.contains(i) ? mainColor : null;
  }

  Offset _getOffetByIndex(int i) {
    var _dxCode = _sizeCode * (i % _col + .5);
    var _dyCode = _sizeCode * ((i / _col).floor() + .5);
    var _offsetCode = Offset(_dxCode, _dyCode);
    return _offsetCode;
  }

  @override
  bool shouldRepaint(_LockScreenPainter oldDelegate) {
    return offset != oldDelegate.offset;
  }
}

class WidgetAppPassPatternUnlock extends StatefulWidget {
  const WidgetAppPassPatternUnlock({Key? key}) : super(key: key);

  @override
  State<WidgetAppPassPatternUnlock> createState() =>
      _WidgetAppPassPatternUnlockState();
}

class _WidgetAppPassPatternUnlockState
    extends State<WidgetAppPassPatternUnlock> {
  PassPatterState patternState = PassPatterState.drawing;
  double valueKey = 2;

  @override
  void initState() {
    super.initState();
    _init();
  }

  _init() async {
    if (!appPrefs.appPassBiometrics) return;
    appPrefs.enableAppPass = false;
    try {
      final bool didAuthenticate = await auth.authenticate(
          localizedReason: 'Please authenticate to continue!',
          options: const AuthenticationOptions(useErrorDialogs: false));
      if (didAuthenticate) {
        oowooHaptic();
        Get.back();
      }
    } on PlatformException catch (e) {
      print(e);
      if (e.code == auth_error.notAvailable) {
        // Add handling of no hardware here.
      } else if (e.code == auth_error.notEnrolled) {
        // ...
      } else {
        // ...
      }
    }
    appPrefs.enableAppPass = true;
  }

  Offset? offset;
  List<int> codes = [];
  _onPanStart(DragStartDetails event) => _clearCodes();

  _onPanUpdate(DragUpdateDetails event) =>
      setState(() => offset = event.localPosition);

  _onPanEndConfirm(DragEndDetails event) {
    setState(() => offset = null);
    Timer(const Duration(milliseconds: 300), () {
      Function eq = const ListEquality().equals;

      if (eq(codes, appPrefs.appPassPattern)) {
        patternState = PassPatterState.correct;
        oowooHaptic();
        Timer(const Duration(milliseconds: 300), () {
          Get.back();
        });
      } else {
        setState(() {
          valueKey = Random().nextDouble();
          patternState = PassPatterState.wrong;
        });
      }
    });
  }

  _onSelect(int code) {
    if (codes.isEmpty || codes.last != code) {
      oowooHaptic();
      codes.add(code);
    }
  }

  _clearCodes() => setState(() {
        patternState = PassPatterState.drawing;
        codes = [];
        offset = null;
      });

  late double sizePainter = Get.width - 32 * 3;

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async => false,
      child: widgetSettingOptions.bottomSheetBuilder(
        // boxshadowsLight: [
        //   BoxShadow(
        //       color: blue.withOpacity(.1),
        //       spreadRadius: 4,
        //       blurRadius: 12)
        // ],
        padding: const EdgeInsets.symmetric(vertical: 40, horizontal: 32),
        child: SizedBox(
          width: Get.width,
          child: Column(
            children: [
              Text(
                'drawapattern'.tr,
                style: assistantMedium.copyWith(fontSize: 16),
              ),
              const SizedBox(
                height: 4,
              ),
              if (patternState == PassPatterState.wrong)
                Text(
                  'wrongpattern'.tr,
                  style: assistantLight.copyWith(fontSize: 12, color: red),
                )
              else
                Text(
                  'minimum4dots'.tr,
                  style: assistantLight.copyWith(fontSize: 12),
                ),
              const SizedBox(
                height: 16,
              ),
              SizedBox(
                height: sizePainter,
                child: GestureDetector(
                  onPanStart: _onPanStart,
                  onPanUpdate: _onPanUpdate,
                  onPanEnd: _onPanEndConfirm,
                  child: CustomPaint(
                    painter: _LockScreenPainter(
                        mainColor: patternState == PassPatterState.drawing
                            ? Colors.white
                            : patternState == PassPatterState.wrong
                                ? red
                                : blue,
                        codes: codes,
                        offset: offset,
                        onSelect: _onSelect),
                    size: Size.square(sizePainter),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}

enum PassPatterState { drawing, wrong, correct }
