import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:oowoo_core/oowoo_core.dart';
 
import 'styles.dart'; 

class DateTimeFormat {
  String format;
  String? _label;

  String get label {
    _label ??= DateFormat(format).format(DateTime.now());
    return _label!;
  }

  DateTimeFormat({required this.format});
}

class WidgetDateTimeFormat extends StatefulWidget {
  const WidgetDateTimeFormat({
    Key? key,
  }) : super(key: key);

  @override
  _WidgetDateTimeFormatState createState() => _WidgetDateTimeFormatState();
}

class _WidgetDateTimeFormatState extends State<WidgetDateTimeFormat> {
  List<DateTimeFormat> timeFormats = [
    DateTimeFormat(format: 'H:mm'),
    DateTimeFormat(format: 'hh:mm aaa')
  ];

  @override
  Widget build(BuildContext context) {
    return widgetSettingOptions.bottomSheetBuilder(
      padding: const EdgeInsets.symmetric(vertical: 20),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: timeFormats.map((e) {
              bool selected = appPrefs.timeFormat == e.format;
              return InkWell(
                onTap: () {
                  oowooHaptic();
                  if (!selected) {
                    appPrefs.timeFormat = e.format;
                    setState(() {});
                  }
                },
                child: Container(
                  padding: const EdgeInsets.symmetric(
                      vertical: 10.0, horizontal: 24),
                  decoration: BoxDecoration(
                    border: Border.all(
                        width: 1,
                        color: selected ? appColors.text : Colors.transparent),
                    borderRadius: BorderRadius.circular(8),
                    color: selected ? Colors.white10 : Colors.transparent,
                  ),
                  child: Text(
                    e.label,
                    style: latoBold.copyWith(fontSize: 18),
                  ),
                ),
              );
            }).toList(),
          ),
        ],
      ),
    );
  }
}

class WidgetDateFormat extends StatefulWidget {
  const WidgetDateFormat({Key? key}) : super(key: key);

  @override
  _WidgetDateFormatState createState() => _WidgetDateFormatState();
}

class _WidgetDateFormatState extends State<WidgetDateFormat> {
  List<DateTimeFormat> dateFormats = [
    DateTimeFormat(format: 'dd MMMM yyyy'),
    DateTimeFormat(format: 'MMMM dd, yyyy'),
    DateTimeFormat(format: 'dd MMMM yy'),
    DateTimeFormat(format: 'dd/MM/yy'),
    DateTimeFormat(format: 'yy/MM/dd'),
  ];

  bool isPressing = false;
  late FixedExtentScrollController scrollController =
      FixedExtentScrollController(
          initialItem:
              dateFormats.indexWhere((e) => e.format == appPrefs.dateFormat));

  @override
  Widget build(BuildContext context) {
    return widgetSettingOptions.bottomSheetBuilder(
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          SizedBox(
            width: Get.width,
            height: 180,
            child: Stack(
              alignment: Alignment.center,
              children: [
                Container(
                  color: Colors.white10,
                  height: 40,
                  width: Get.width,
                ),
                CupertinoPicker(
                  scrollController: scrollController,
                  selectionOverlay:
                      const CupertinoPickerDefaultSelectionOverlay(
                    background: Colors.transparent,
                    capEndEdge: false,
                    capStartEdge: false,
                  ),
                  itemExtent: 50,
                  onSelectedItemChanged: (int index) {
                    appPrefs.dateFormat = dateFormats[index].format;
                  },
                  backgroundColor: Colors.transparent,
                  children: dateFormats
                      .map(
                        (e) => Center(
                          child: Text(
                            e.label,
                            style: latoBold.copyWith(fontSize: 17),
                          ),
                        ),
                      )
                      .toList(),
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
